@@ -1,5 +1,5 @@
 /** BEGIN COPYRIGHT BLOCK
- * Copyright (C) 2020 Red Hat, Inc.
+ * Copyright (C) 2025 Red Hat, Inc.
  * All rights reserved.
  *
  * License: GPL (version 3 or any later version).
@@ -311,6 +311,7 @@ int dbmdb_ctx_t_load_dse_info(struct ldbminfo *li);
 int dbmdb_ctx_t_internal_set(struct ldbminfo *li, char *attrname, char *value);
 void dbmdb_public_config_get(struct ldbminfo *li, char *attrname, char *value);
 int dbmdb_public_config_set(struct ldbminfo *li, char *attrname, int apply_mod, int mod_op, int phase, char *value);
+uint32_t dbmdb_get_inst_page_count(struct ldbminfo *li, ldbm_instance *inst);
 
 /* dbimpl callbacks */
 dblayer_get_db_filename_fn_t dbmdb_public_get_db_filename;
@@ -394,8 +395,6 @@ void dbmdb_restore_file_update(struct ldbminfo *li, const char *directory);
 int dbmdb_import_file_init(ldbm_instance *inst);
 void dbmdb_import_file_update(ldbm_instance *inst);
 int dbmdb_import_file_check(ldbm_instance *inst);
-int dbmdb_import_subcount_mother_init(import_subcount_stuff *mothers, ID parent_id, size_t count);
-int dbmdb_import_subcount_mother_count(import_subcount_stuff *mothers, ID parent_id);
 void dbmdb_import_configure_index_buffer_size(size_t size);
 size_t dbmdb_import_get_index_buffer_size(void);
 int dbmdb_ldbm_back_wire_import(Slapi_PBlock *pb);
